
#include "unix.h"
#include "x.h"
#include "xt.h"
#include "xmu.h"
#include "xpex.h"
#include "color.h"
#include "externs.h"
#include "globals.h"

extern XtAppContext app_context;
extern Display     *display;
extern Colormap     cmap;
extern XColor       colors[64];
extern GC  boxedGC, boxedChangedGC, unboxedGC;

extern int bars_locked;
extern Boolean do_change;
extern Pixel original_background;
extern float pass_value;

extern Widget toplevel, mixingForm, mixedColor;
extern Widget rgb, hsv, hls, cie;

extern Widget redLocked, greenLocked, blueLocked, title;
extern Widget redLabel, greenLabel, blueLabel;
extern Widget redIntensity, greenIntensity, blueIntensity, tripleValue;
extern Widget redScroll, greenScroll, blueScroll, lockedScroll;
extern Widget hueScroll, satScroll, valScroll;
extern Widget hueLabel, satLabel, valLabel;
extern Widget quit;

extern Boolean workingRGB, workingHSV, workingHLS, workingCIE;

void makeRGB_Color( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    Widget popup;
    Widget widget = w;
    XGCValues values;
    Cursor cursor;
    extern PEXColorSpecifier colorSpec;

    XtSetSensitive( rgb, False );    workingRGB = True;
                                     colorSpec.type = PEXColorTypeRGB;
    XtSetSensitive( hsv, False );    workingHSV = False;
    XtSetSensitive( hls, False );    workingHLS = False;
    XtSetSensitive( cie, False );    workingCIE = False;

    cursor = XCreateFontCursor( display, XC_sb_v_double_arrow );

    popup = XtVaCreateManagedWidget( "rgbColorEditor",
        topLevelShellWidgetClass, toplevel,
        XtNwidth, 307,
        XtNheight, 375,
        NULL );
    mixingForm = XtVaCreateManagedWidget( "mixingForm", formWidgetClass, popup,
        NULL );
    title = XtVaCreateManagedWidget( "title", labelWidgetClass, mixingForm,
        XtNlabel, "Choose RGB Color",
        NULL );
    mixedColor = XtVaCreateManagedWidget( "mixedColor", labelWidgetClass,
        mixingForm,
        XtNlabel, "",
        XtNwidth, 202,
        XtNheight, 192,
        XtNhorizDistance, 33,
        XtNbackground, colors[MIX_COLOR].pixel,
        XtNfromVert, title,
        NULL );
    redIntensity = XtVaCreateManagedWidget( "redIntensity", widgetClass,
        mixingForm,
        XtNbackground, colors[RED_COLOR].pixel,
        NULL );
    greenIntensity = XtVaCreateManagedWidget( "greenIntensity", widgetClass,
        mixingForm,
        XtNbackground, colors[GREEN_COLOR].pixel,
        NULL );
    blueIntensity = XtVaCreateManagedWidget( "blueIntensity", widgetClass,
        mixingForm,
        XtNbackground, colors[BLUE_COLOR].pixel,
        NULL );
    redLocked = XtVaCreateManagedWidget( "redLocked", labelWidgetClass,
        mixingForm, NULL );
    greenLocked = XtVaCreateManagedWidget( "greenLocked", labelWidgetClass,
        mixingForm,
        NULL );
    blueLocked = XtVaCreateManagedWidget( "blueLocked", labelWidgetClass,
        mixingForm, NULL );
    redScroll = XtVaCreateManagedWidget( "redScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollHCursor, cursor,
        NULL );
    redLabel = XtVaCreateManagedWidget( "redLabel", labelWidgetClass,
        mixingForm, NULL );
    greenScroll = XtVaCreateManagedWidget( "greenScroll",
        scrollbarWidgetClass, mixingForm,
        XtNscrollHCursor, cursor,
        NULL );
    greenLabel = XtVaCreateManagedWidget( "greenLabel", labelWidgetClass,
        mixingForm, NULL );
    blueScroll = XtVaCreateManagedWidget( "blueScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollHCursor, cursor,
        NULL );
    blueLabel = XtVaCreateManagedWidget( "blueLabel", labelWidgetClass,
        mixingForm, NULL );
    lockedScroll = XtVaCreateManagedWidget( "lockedScroll",
        scrollbarWidgetClass, mixingForm,
        XtNscrollHCursor, cursor,
        NULL );
    quit = XtVaCreateManagedWidget( "quit", commandWidgetClass, mixingForm,
        XtNfromVert, lockedScroll,
        NULL );

    tripleValue = XtVaCreateManagedWidget( "tripleValue", toggleWidgetClass,
        mixingForm,
        XtNfromHoriz, title,
        XtNwidth, 70,
        XtNhorizDistance, 91,
        NULL );

    values.foreground = colors[PURE_RED].pixel;
    values.line_width = 2;
    values.line_style = LineOnOffDash;
    boxedChangedGC = XtGetGC( mixingForm, GCForeground | GCLineWidth,
        &values );
    boxedGC = XtGetGC( mixingForm, GCForeground | GCLineWidth | GCLineStyle,
        &values );

    XtVaGetValues( mixingForm, XtNbackground, &( values.foreground ), NULL );
    unboxedGC = XtGetGC( mixingForm, GCForeground | GCLineWidth, &values );

    original_background = values.foreground;

    bars_locked = 0;

    XtAddCallback( redScroll, XtNjumpProc, Thumbed, ( XtPointer )RED );
    XtAddCallback( greenScroll, XtNjumpProc, Thumbed, ( XtPointer )GREEN );
    XtAddCallback( blueScroll, XtNjumpProc, Thumbed, ( XtPointer )BLUE );
    XtAddCallback( lockedScroll, XtNjumpProc, Thumbed, ( XtPointer )LOCKED );

    XtAddCallback( redScroll, XtNscrollProc, Scrolled, ( XtPointer )RED );
    XtAddCallback( greenScroll, XtNscrollProc, Scrolled, ( XtPointer )GREEN );
    XtAddCallback( blueScroll, XtNscrollProc, Scrolled, ( XtPointer )BLUE );
    XtAddCallback( lockedScroll, XtNscrollProc, Scrolled, ( XtPointer )LOCKED );

    XtAddCallback( tripleValue, XtNcallback, Set_Selection,
                  ( XtPointer )PRIMARY_COLOR );
    XtAddCallback( quit, XtNcallback, Quit, ( XtPointer )toplevel );

    /* set the scrollbars to the initial mixed colour values */
    do_change = FALSE;
    CHANGE_RED( MIX_COLOR );
    CHANGE_GREEN( MIX_COLOR );
    CHANGE_BLUE( MIX_COLOR );
    do_change = TRUE;
    XStoreColors( display, cmap, colors, 4 );

    XtRealizeWidget( popup );
    XtPopup( popup, XtGrabNone );
}
